package com.zy.bms.handler;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zy.bms.common.Constant;
import com.zy.bms.common.exception.ApiRuntimeException;
import com.zy.bms.entity.*;
import com.zy.bms.entity.ubi.UbiNtrip;
import com.zy.bms.entity.ubi.UbiInfoRecord;
import com.zy.bms.entity.ubi.UbiInfoState;
import com.zy.bms.entity.ubi.UbiTcp;
import com.zy.bms.pojo.dto.MqttDTO;
import com.zy.bms.service.*;
import com.zy.bms.service.ubi.IUbiInfoRecordService;
import com.zy.bms.service.ubi.IUbiInfoStateService;
import com.zy.bms.service.ubi.IUbiNtripService;
import com.zy.bms.service.ubi.IUbiTcpService;
import com.zy.bms.utils.BeanUtil;
import com.zy.bms.utils.MD5Util;
import com.zy.bms.utils.RabbitMQApi;
import com.zy.bms.utils.TransformEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Map;

/**
 * MQTT 消息返回处理类
 *
 * @author yang xiao kun
 * create on 2021/1/19
 */
@Slf4j
@Component
public class MqttCallbackHandler {

    @Resource
    private WebSocketHandler webSocketHandler;
    @Resource
    private IPostMqttMsgService postMqttMsgService;
    @Resource
    private ILogsService logsService;
    @Resource
    private IDeviceBaseService deviceStaticService;
    @Resource
    private IUbiInfoRecordService heartbeatRecordService;
    @Resource
    private IUbiInfoStateService deviceDynamicService;
    @Resource
    private IUbiNtripService ntripSetService;
    @Resource
    private IUbiTcpService tcpSetService;

    /**
     * 处理消息
     *
     * @param topic   主题
     * @param payload 消息内容
     */
    public void handle(String topic, String payload) {
        //设备码
        String openNum = Constant.splitOpenNum(topic);
        MqttDTO mqttDTO;
        try {
            //解析消息
            mqttDTO = MqttDTO.parse(payload);
        } catch (Exception e) {
            log.error("mqtt消息格式异常:" + payload, e);
            return;
        }
        // 1. 存储日志 --- 接收
        logsService.saveReceive(topic, payload);
        // 2. 根据报文标识代码 M 处理消息
        switch (mqttDTO.getM()) {
            //设备端注册报文
            case Constant.M.REGISTER: {
                registerHandler(mqttDTO);
                break;
            }
            // 心跳包
            case Constant.M.UBI_HEARTBEAT: {
                heartbeatHandler(openNum, mqttDTO);
                break;
            }
            // 获取设备设置信息
            case Constant.M.UBI_SET_INFO: {
                deviceSetHandler(openNum, mqttDTO);
                break;
            }
        }
        // 3. 转发消息至网页,忽略注册的报文
        webSocketHandler.massMessage(Constant.splitOpenNum(topic), payload);
    }

    /**
     * 注册设备
     */
    private void registerHandler(MqttDTO mqttDTO) {
        String openNum = mqttDTO.getString("regnum");
        String group = mqttDTO.getString("group");
        // 查询数据库中是否有此设备
        DeviceBase device = deviceStaticService
                .getOne(new QueryWrapper<DeviceBase>()
                        .eq("open_num", openNum));
        // 没有该设备,保存至数据库
        if (device == null) {
            device = new DeviceBase();
            device.setOpenNum(openNum);
            device.setGroupId(group);
            device.setUsername("IMEI" + openNum);
            device.setPassword(MD5Util.encode(openNum));
            device.setSn("SN" + openNum);
            deviceStaticService.save(device);
        }
        // mqtt服务器注册用户
        RabbitMQApi.register(device.getUsername(), device.getPassword());
        // 返回处理结果
        postMqttMsgService.handleRegister(device);
    }

    /**
     * 心跳协议
     */
    private void heartbeatHandler(String openNum, MqttDTO mqttDTO) {
        try {
            UbiInfoRecord entity = TransformEntity.tsf2UbiInfoRecord(openNum, mqttDTO);
            //存储记录
            heartbeatRecordService.save(entity);
            //更新最新的记录
            UbiInfoState ubiInfoState = BeanUtil.cast(entity, UbiInfoState.class);
            //设置更新时间
            ubiInfoState.setUpdateTime(LocalDateTime.now());
            //设置在线状态
            ubiInfoState.setStatus(1);
            deviceDynamicService.saveOrUpdate(ubiInfoState,
                    new QueryWrapper<UbiInfoState>().eq("open_num", ubiInfoState.getOpenNum()));
        } catch (Exception e) {
            log.error("解析设备心跳消息异常", e);
            throw new ApiRuntimeException("处理设备上传位置定位信息出错");
        }
    }

    /**
     * 普适性-设备设置信息
     */
    @SuppressWarnings("unchecked")
    private void deviceSetHandler(String openNum, MqttDTO mqttDTO) {
        // ntrip数据链路设置
        UbiNtrip client = TransformEntity.tsf2UbiNtrip((Map<String, String>) mqttDTO.getObject("ntripclient"));
        UbiNtrip server = TransformEntity.tsf2UbiNtrip((Map<String, String>) mqttDTO.getObject("ntripserver"));
        ntripSetService.saveOrUpdateNtripSet(client, openNum, IUbiNtripService.CLIENT);
        ntripSetService.saveOrUpdateNtripSet(server, openNum, IUbiNtripService.SERVER);
        // tcp数据链路设置
        UbiTcp link1 = TransformEntity.tsf2UbiTcp((Map<String, String>) mqttDTO.getObject("tcplink1"));
        UbiTcp link2 = TransformEntity.tsf2UbiTcp((Map<String, String>) mqttDTO.getObject("tcplink2"));
        UbiTcp link3 = TransformEntity.tsf2UbiTcp((Map<String, String>) mqttDTO.getObject("tcplink3"));
        tcpSetService.saveOrUpdateTcpSet(link1, openNum, IUbiTcpService.LINK_1);
        tcpSetService.saveOrUpdateTcpSet(link2, openNum, IUbiTcpService.LINK_2);
        tcpSetService.saveOrUpdateTcpSet(link3, openNum, IUbiTcpService.LINK_3);
    }
}
